
#include "opt/config.h"

using std::make_pair;
using std::pair;
using std::string;
using std::vector;

struct OptConfig::M_Data {

  M_Data() {

  }

  M_Data(const M_Data &data) :
    binaryProperties(data.binaryProperties),
    intProperties(data.intProperties),
    doubleProperties(data.doubleProperties)
  {
    
  }

  ~M_Data() {

  }

  vector<string> binaryProperties;
  vector<pair<string,int> > intProperties;
  vector<pair<string,double> > doubleProperties;
};

OptConfig::OptConfig() : m_data(new M_Data())
{

}

OptConfig::OptConfig(const OptConfig &config) :
  m_data(new M_Data(*config.m_data))
{

}

OptConfig::~OptConfig() 
{
  delete m_data;
}

OptConfig &
OptConfig::operator=(const OptConfig &config)
{
  if (&config != this) {
    delete m_data;
    m_data = new M_Data(*config.m_data);
  }

  return *this;
}

void
OptConfig::SetProperty(const string &property)
{
  m_data->binaryProperties.push_back(property);
}

void
OptConfig::SetProperty(const string &property,
		       int value)
{
  m_data->intProperties.push_back(make_pair(property,value));
}

void
OptConfig::SetProperty(const string &property,
		       double value)
{
  m_data->doubleProperties.push_back(make_pair(property,value));
}

const vector<string> &
OptConfig::GetBinaryProperties() const
{
  return m_data->binaryProperties;
}

const vector<pair<string,int> > &
OptConfig::GetIntProperties() const
{
  return m_data->intProperties;
}

const vector<pair<string,double> > &
OptConfig::GetDoubleProperties() const
{
  return m_data->doubleProperties;
}
