#ifndef _OPT_EQUATION_H_
#define _OPT_EQUATION_H_

#include <vector>

class OptEquation {
 public:

  virtual int GetRangeDimension() const = 0;

  virtual unsigned int GetNumberOfLinearCoefficients() const = 0;

  virtual const std::vector<int> &
    GetLinearCoefficientFunctionIndices() const = 0;
  
  virtual const std::vector<int> & 
    GetLinearCoefficientVariableIndices() const = 0;
  
  virtual const double *GetLinearCoefficients() const = 0;

  virtual unsigned int GetGradientSize() const = 0;

  virtual const std::vector<int> &GetGradientFunctionIndices() const = 0;

  virtual const std::vector<int> &GetGradientVariableIndices() const = 0;
  
  // give the equation a chance to cache some stuff
  virtual void SetState(const double *x) = 0;
  
  // offset is where in the array to start placing results
  virtual void Evaluate(unsigned int offset,
			double *f) const = 0;

  virtual void Gradient(unsigned int offset,
			double *g) const = 0;
			
};

#endif // _OPT_EQUATION_H_
