#ifndef _OPT_PROBLEM_H_
#define _OPT_PROBLEM_H_

#include <string>

class OptProblem {
 public:

  virtual int GetNumberOfEquations() const = 0;
  
  virtual int GetNumberOfVariables() const = 0;

  virtual int GetNumberOfLinearCoefficients() const = 0;

  virtual int GetGradientSize() const = 0;
  
  virtual int GetHessianSize() const = 0;

  virtual const int *GetLinearCoefficientFunctionIndices() const = 0;
  
  virtual const int *GetLinearCoefficientVariableIndices() const = 0;

  virtual const double *GetLinearCoefficients() const = 0;

  virtual const int *GetGradientFunctionIndices() const = 0;
  
  virtual const int *GetGradientVariableIndices() const = 0;
  
  virtual const int *GetHessianFunctionIndices() const = 0;

  virtual const int *GetHessianVariableIndices() const = 0;

  virtual const double *GetVariableLowerBounds() const = 0;

  virtual const double *GetVariableUpperBounds() const = 0;

  virtual const double *GetEquationLowerBounds() const = 0;

  virtual const double *GetEquationUpperBounds() const = 0;

  virtual const std::string &GetName() const = 0;

  virtual void SetState(const double *x) = 0;
  
  virtual void Evaluate(double *f) const = 0;

  virtual void Gradient(double *G) const = 0;
};
#endif
