#ifndef _OPT_PROBLEM_BASE_H_
#define _OPT_PROBLEM_BASE_H_

#include "opt/problem.h"
#include <vector>

#define OPT_BIG_BND 1e30

class OptEquation;

class OptProblemBase : public OptProblem {
  
 public:
  
  OptProblemBase(const std::string &name,
		 const std::vector<OptEquation*> &equations,
		 int numberOfVariables,
		 const std::vector<double> &variableLowerBounds,
		 const std::vector<double> &variableUpperBounds,
		 const std::vector<double> &equationLowerBounds,
		 const std::vector<double> &equationUpperBounds);

  virtual ~OptProblemBase();		 

  virtual int GetNumberOfEquations() const;
  
  virtual int GetNumberOfVariables() const;

  virtual int GetNumberOfLinearCoefficients() const;

  virtual int GetGradientSize() const;
  
  virtual int GetHessianSize() const;

  virtual const int *GetLinearCoefficientFunctionIndices() const;
  
  virtual const int *GetLinearCoefficientVariableIndices() const;

  virtual const double *GetLinearCoefficients() const;

  virtual const int *GetGradientFunctionIndices() const;
  
  virtual const int *GetGradientVariableIndices() const;
  
  virtual const int *GetHessianFunctionIndices() const;

  virtual const int *GetHessianVariableIndices() const;

  virtual const double *GetVariableLowerBounds() const;

  virtual const double *GetVariableUpperBounds() const;

  virtual const double *GetEquationLowerBounds() const;

  virtual const double *GetEquationUpperBounds() const;

  virtual const std::string &GetName() const;

  virtual void SetState(const double *x);
  
  virtual void Evaluate(double *f) const;

  virtual void Gradient(double *G) const;

 private:

  struct M_Data;

  M_Data *m_data;
};

#endif // #ifndef opt problem base
